// Written by: George Ng and Pearl Liu
// ngg@ecf.utoronto.ca liupea@ecf.utoronto.ca
// March 23, 2005
// Top level hardware design for the ViewSVGAbmp_orig project
// Design uses ZBT RAM BANK 2 for display controller and ZBT BANK 0 as data source

module displaypattern(
   // connections to the video DAC
   VGA_OUT_PIXEL_CLOCK_P,
   VGA_COMP_SYNCH_N,
   VGA_OUT_BLANK_N,
   VGA_HSYNCH_N,
   VGA_VSYNCH_N,
   VGA_OUT_RED_P,
   VGA_OUT_GREEN_P,
   VGA_OUT_BLUE_P,

   // connections to the ZBT RAM BANK 2
   MEMORY_CLK_P,
   MEMORY_CLKEN_N,
   MEMORY_WEN_N,
   MEMORY_WENA_N,
   MEMORY_WENB_N,
   MEMORY_WENC_N,
   MEMORY_WEND_N,
   MEMORY_ADV_LD_N,
   MEMORY_OEN_N,
   MEMORY_CEN_N,
   MEMORY_ADDR_P,
   MEMORY_DATA_A_P,
   MEMORY_DATA_B_P,
   MEMORY_DATA_C_P,
   MEMORY_DATA_D_P,
   
   // connections to the ZBT RAM BANK 0
   MEMORY_0_CLK_P,
   MEMORY_0_CLKEN_N,
   MEMORY_0_WEN_N,
   MEMORY_0_WENA_N,
   MEMORY_0_WENB_N,
   MEMORY_0_WENC_N,
   MEMORY_0_WEND_N,
   MEMORY_0_ADV_LD_N,
   MEMORY_0_OEN_N,
   MEMORY_0_CEN_N,
   MEMORY_0_ADDR_P,
   MEMORY_0_DATA_A_P,
   MEMORY_0_DATA_B_P,
   MEMORY_0_DATA_C_P,
   MEMORY_0_DATA_D_P,
   
   pixel_clock, //aka system clock (look in ucf)
   reset
);

output         VGA_OUT_PIXEL_CLOCK_P;  // pixel clock for the video DAC
output         VGA_COMP_SYNCH_N;    // composite sync for the video DAC
output         VGA_OUT_BLANK_N;     // composite blanking for the video DAC
output         VGA_HSYNCH_N;     // horizontal sync for the VGA output connector
output         VGA_VSYNCH_N;     // vertical sync for the VGA output connector
output [7:0]   VGA_OUT_RED_P;    // RED DAC data
output [7:0]   VGA_OUT_GREEN_P;     // GREEN DAC data
output [7:0]   VGA_OUT_BLUE_P;      // BLUE DAC data

// ZBT RAM BANK 2
output [18:0]  MEMORY_ADDR_P;    // external memory address bus
output         MEMORY_CLK_P;     // external memory clock
output         MEMORY_CLKEN_N;      // external memory clock enable
output         MEMORY_WEN_N;     // external memory read/write
output         MEMORY_WENA_N;    // external memory write enable for byte A
output         MEMORY_WENB_N;    // external memory write enable for byte B
output         MEMORY_WENC_N;    // external memory write enable for byte C
output         MEMORY_WEND_N;    // external memory write enable for byte D
output         MEMORY_OEN_N;     // external memory output enable
output         MEMORY_CEN_N;     // external memory chip enable
output         MEMORY_ADV_LD_N;     // external memory load/advance
inout [7:0]    MEMORY_DATA_A_P;     // external memory byte A bits [ 7: 0] BLUE DATA
inout [7:0]    MEMORY_DATA_B_P;     // external memory byte B bits [15: 8] GREEN DATA
inout [7:0]    MEMORY_DATA_C_P;     // external memory byte C bits [23:16] RED DATA
inout [7:0]    MEMORY_DATA_D_P;     // external memory byte D bits [31:24] MISC DATA

// ZBT RAM BANK 0
output [18:0]  MEMORY_0_ADDR_P; 
output         MEMORY_0_CLK_P;  
output         MEMORY_0_CLKEN_N;
output         MEMORY_0_WEN_N;  
output         MEMORY_0_WENA_N; 
output         MEMORY_0_WENB_N; 
output         MEMORY_0_WENC_N; 
output         MEMORY_0_WEND_N; 
output         MEMORY_0_OEN_N;  
output         MEMORY_0_CEN_N;  
output         MEMORY_0_ADV_LD_N;
inout [7:0]    MEMORY_0_DATA_A_P;
inout [7:0]    MEMORY_0_DATA_B_P;
inout [7:0]    MEMORY_0_DATA_C_P;
inout [7:0]    MEMORY_0_DATA_D_P;

input       pixel_clock          /* synthesis syn_noclockbuf = 1 */; // pixel clock input 
input       reset;               // reset

wire           VGA_OUT_PIXEL_CLOCK_P;
wire           VGA_COMP_SYNCH_N;
wire           VGA_OUT_BLANK_N;
wire           VGA_HSYNCH_N;
wire           VGA_VSYNCH_N;
wire  [7:0]    VGA_OUT_RED_P; 
wire  [7:0]    VGA_OUT_GREEN_P; 
wire  [7:0]    VGA_OUT_BLUE_P;

wire  [18:0]   MEMORY_ADDR_P;
wire           MEMORY_CLK_P;
wire           MEMORY_CLKEN_N;
wire           MEMORY_WEN_N;
wire           MEMORY_WENA_N;
wire           MEMORY_WENB_N;
wire           MEMORY_WENC_N;
wire           MEMORY_WEND_N;
wire           MEMORY_OEN_N;
wire           MEMORY_CEN_N;
wire           MEMORY_ADV_LD_N;
wire   [7:0]   MEMORY_DATA_A_P;
wire   [7:0]   MEMORY_DATA_B_P;
wire   [7:0]   MEMORY_DATA_C_P;
wire   [7:0]   MEMORY_DATA_D_P;

wire   [18:0]  MEMORY_0_ADDR_P; 
wire           MEMORY_0_CLK_P;  
wire           MEMORY_0_CLKEN_N;
wire           MEMORY_0_WEN_N;  
wire           MEMORY_0_WENA_N; 
wire           MEMORY_0_WENB_N; 
wire           MEMORY_0_WENC_N; 
wire           MEMORY_0_WEND_N; 
wire           MEMORY_0_OEN_N;  
wire           MEMORY_0_CEN_N;  
wire           MEMORY_0_ADV_LD_N;
wire [7:0]     MEMORY_0_DATA_A_P;
wire [7:0]     MEMORY_0_DATA_B_P;
wire [7:0]     MEMORY_0_DATA_C_P;
wire [7:0]     MEMORY_0_DATA_D_P;

reg   [18:0]   user_vga_addr;
wire  [31:0]   user_vga_write_data;

wire fifo_full;
wire fifo_empty;
wire [31:0] fifo_dout;
wire [18:0] mem_0_addr;
wire fifo_wr_en;
wire fifo_read_en;
wire data_valid;
wire [31:0] user_read_data;
wire last_data;

BM_MODE_SVGA_CTRL user_BM_MODE_SVGA_CTRL (
   // connections to the DAC
   .VGA_OUT_PIXEL_CLOCK_P(VGA_OUT_PIXEL_CLOCK_P),
   .VGA_COMP_SYNCH_N(VGA_COMP_SYNCH_N),
   .VGA_OUT_BLANK_N(VGA_OUT_BLANK_N),
   .VGA_HSYNCH_N(VGA_HSYNCH_N),
   .VGA_VSYNCH_N(VGA_VSYNCH_N),
   .VGA_OUT_RED_P(VGA_OUT_RED_P),
   .VGA_OUT_GREEN_P(VGA_OUT_GREEN_P),
   .VGA_OUT_BLUE_P(VGA_OUT_BLUE_P),

   // connections to the ZBT RAM
   .MEMORY_CLK_P(MEMORY_CLK_P),
   .MEMORY_CLKEN_N(MEMORY_CLKEN_N),
   .MEMORY_WEN_N(MEMORY_WEN_N),
   .MEMORY_WENA_N(MEMORY_WENA_N),
   .MEMORY_WENB_N(MEMORY_WENB_N),
   .MEMORY_WENC_N(MEMORY_WENC_N),
   .MEMORY_WEND_N(MEMORY_WEND_N),
   .MEMORY_ADV_LD_N(MEMORY_ADV_LD_N),
   .MEMORY_OEN_N(MEMORY_OEN_N),
   .MEMORY_CEN_N(MEMORY_CEN_N),
   .MEMORY_ADDR_P(MEMORY_ADDR_P),
   .MEMORY_DATA_A_P(MEMORY_DATA_A_P),
   .MEMORY_DATA_B_P(MEMORY_DATA_B_P),
   .MEMORY_DATA_C_P(MEMORY_DATA_C_P),
   .MEMORY_DATA_D_P(MEMORY_DATA_D_P),

   // connections to the user interface
   .user_vga_addr(user_vga_addr),
   .user_vga_read(user_vga_read),
   .user_vga_write(user_vga_write),
   .user_vga_write_byte_a(user_vga_write_byte_a),
   .user_vga_write_byte_b(user_vga_write_byte_b),
   .user_vga_write_byte_c(user_vga_write_byte_c),
   .user_vga_write_byte_d(user_vga_write_byte_d),
   .user_vga_enable(user_vga_enable),
   .user_vga_read_data(user_vga_read_data),
   .user_vga_write_data(user_vga_write_data),
   .user_access_ok(user_access_ok),
   .user_clock_enable(user_clock_enable),
   .user_load_vga_addr(user_load_vga_addr),
   .pixel_clock(pixel_clock),
   .reset(reset)
);

ZBT_CONTROL uZBT_CONTROL_RAM_0 (
   .ram_clock(MEMORY_0_CLK_P), //output clock from controller to RAM
   .ram_clocken_n(MEMORY_0_CLKEN_N),
   .ram_we_n(MEMORY_0_WEN_N),
   .ram_oe_n(MEMORY_0_OEN_N),
   .ram_ce0_n(MEMORY_0_CEN_N),
   .ram_ce1(), //already tied on the board
   .ram_ce2_n(), //already tied on the board
   .ram_adv_ld(MEMORY_0_ADV_LD_N),
   .ram_wea_n(MEMORY_0_WENA_N),
   .ram_web_n(MEMORY_0_WENB_N),
   .ram_wec_n(MEMORY_0_WENC_N),
   .ram_wed_n(MEMORY_0_WEND_N),
   .ram_lbo(), //already tied on the board
   .ram_zz(), //already tied on the board
   .ram_addr(MEMORY_0_ADDR_P),
   .ram_data_bus({MEMORY_0_DATA_D_P,MEMORY_0_DATA_C_P,MEMORY_0_DATA_B_P,MEMORY_0_DATA_A_P}),

   //user signals
   .user_read_data(user_read_data), //RAM data output from controller
   .user_ram_clock(pixel_clock), //connect the RAM to system clock
   .user_clocken(1'b1),
   .user_read_writez(1'b1), //1'b1 is a read, 1'b0 is a write
   .user_oe(1'b1),
   .user_ce0(1'b1),
   .user_ce1(1'b1),  //uneeded since tied on the board
   .user_ce2(1'b1),  //uneeded since tied on the board
   .user_adv_ld(1'b1),
   .user_wea(1'b1),
   .user_web(1'b1),
   .user_wec(1'b1),
   .user_wed(1'b1),
   .user_lbo(1'b1), //uneeded since tied on the board
   .user_zz(1'b0),  //uneeded since tied on the board
   .clock(pixel_clock), //connect controller logic to system clock
   .user_write_data(32'h0),
   .user_ram_addr(mem_0_addr)
);

// Generates controls signals for ZBT BANK 0, filter, and FIFO
mem2fifo_ctrl umem2fifo_ctrl_0 (
   .clk(pixel_clock),
   .reset(reset),
   .full(fifo_full),
   .empty(fifo_empty),
   .mem_addr(mem_0_addr),
   .wr_en(fifo_wr_en),
   .last_data(last_data)
);

// Generates control signals for FIFO and display controller
fifo2disp_ctrl ufifo2disp_ctrl (  
   .clk(pixel_clock),
   .reset(reset),
   .full(fifo_full),     
   .empty(fifo_empty),
   .user_access_ok(user_access_ok),
   .last_data(last_data),
   .read_en(fifo_read_en),
   .data_valid(data_valid)
);

// CoreGen FIFO
mem_fifo umem_fifo_0 (
   .clk(pixel_clock),
   .sinit(reset),
   .din(user_read_data),
   .wr_en(fifo_wr_en),
   .rd_en(fifo_read_en),
   .dout(fifo_dout),
   .full(fifo_full),
   .empty(fifo_empty)
);



// fifo_dout[0:7]   green
// fifo_dout[15:8]  blue
// fifo_dout[31:24] red
assign user_vga_write_data = {fifo_dout[23:16],fifo_dout[31:24],fifo_dout[7:0],fifo_dout[15:8]};

// increment address to draw screen
always@(posedge pixel_clock) begin
   if(reset)
      user_vga_addr <= 19'b0;
   else if(user_vga_addr == 19'h4B000) //at 640x480 bottom right corner
      user_vga_addr <= 19'h4B000;    //stop counting once image is displayed
   else if(user_access_ok && data_valid)
      user_vga_addr <= user_vga_addr + 1;
end

// always enable the VGA controller for display
assign user_vga_read = 1'b0; // don't read from display controller RAM
assign user_vga_write = 1'b1; //always write to display controller RAM
assign user_vga_write_byte_a = 1'b1; //blue
assign user_vga_write_byte_b = 1'b1; //green
assign user_vga_write_byte_c = 1'b1; //red
assign user_vga_write_byte_d = 1'b1; //unused
assign user_vga_enable = 1'b1; //enable use of VGA
assign user_clock_enable = 1'b1; //enable zbt controller clock
assign user_load_vga_addr = 1'b1; //enable address to be loaded into svga controller


endmodule