//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//     (c) Copyright 2002 Xilinx, Inc.
//     All rights reserved.
//

/*
-------------------------------------------------------------------------------
   Title      : Control bus intreface for ZBT Controller
   Project    : MicroBlaze Demo Board
-------------------------------------------------------------------------------
   File       : CTRL_BUS_INTERFACE.v
   Company    : Xilinx, Inc.
   Created    : 2001/06/19
   Last Update: 2001/06/19
   Copyright  : (c) Xilinx Inc, 2001
-------------------------------------------------------------------------------
   Uses       : 
-------------------------------------------------------------------------------
   Used by    : ZBT_CONTROL.v
-------------------------------------------------------------------------------
   Description: This module defines the control bus interface for the ZBT RAM 
		    controller. There is a two level pipeline for the output enable
		    because the ZBT RAM does not resister this control signal.
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
*/

module CTRL_BUS_INTERFACE (
ram_clocken_n,
ram_we_n,
ram_oe_n,
ram_ce0_n,
ram_ce1,
ram_ce2_n,
ram_adv_ld,
ram_wea_n,
ram_web_n,
ram_wec_n,
ram_wed_n,
ram_lbo,
ram_zz,

user_clocken,
user_read_writez,
user_oe,
user_ce0,
user_ce1,
user_ce2,
user_adv_ld,
user_wea,
user_web,
user_wec,
user_wed,
user_lbo,
user_zz,
clock
)/* synthesis syn_useioff = 1*/;

// interfaces to user logic
input 	user_clocken;
input 	user_read_writez;
input 	user_oe;
input 	user_ce0;
input 	user_ce1;
input 	user_ce2;
input 	user_adv_ld;
input 	user_wea;
input 	user_web;
input 	user_wec;
input 	user_wed;
input 	user_lbo;
input 	user_zz;
input 	clock;

// Interface to ZBT SRAM
output	ram_clocken_n;
output	ram_we_n;
output	ram_oe_n;
output	ram_ce0_n;
output	ram_ce1;
output	ram_ce2_n;
output	ram_adv_ld;
output	ram_wea_n;
output	ram_web_n;
output	ram_wec_n;
output	ram_wed_n;
output	ram_lbo;
output	ram_zz; 

wire		ram_clocken;
wire		ram_we;
wire		ram_oe;
wire		ram_ce0;
wire		ram_ce1;
wire		ram_ce2;
wire		ram_adv_ld;
wire		ram_wea;
wire		ram_web;
wire		ram_wec;
wire		ram_wed;
wire		ram_lbo;
wire		ram_zz; 

reg		latched_user_clocken;
reg		latched_user_read_writez;
reg		latched_user_oe;
reg		oe_pipe1;
reg		oe_pipe2;
reg		latched_user_ce0;
reg		latched_user_ce1;
reg		latched_user_ce2;
reg		latched_user_adv_ld;
reg		latched_user_wea;
reg		latched_user_web;
reg		latched_user_wec;
reg		latched_user_wed;
reg		latched_user_lbo;
reg		latched_user_zz;

always @ (posedge clock) begin
	latched_user_clocken 		<= user_clocken;
	latched_user_read_writez 	<= user_read_writez;
	latched_user_oe 			<= user_oe;
	oe_pipe1					<= latched_user_oe;
	oe_pipe2					<= oe_pipe1;
	latched_user_ce0 			<= user_ce0;
	latched_user_ce1 			<= user_ce1;
	latched_user_ce2 			<= user_ce2;
	latched_user_adv_ld 		<= user_adv_ld;
	latched_user_wea 			<= user_wea;
	latched_user_web 			<= user_web;
	latched_user_wec 			<= user_wec;
	latched_user_wed 			<= user_wed;
	latched_user_lbo 			<= user_lbo;
	latched_user_zz 			<= user_zz;
end

// instantiate the output buffers
OBUF_F_12 I_obuf_ram_clocken	(.O(ram_clocken_n),	.I(!latched_user_clocken));
OBUF_F_12 I_obuf_ram_we 		(.O(ram_we_n), 	.I(latched_user_read_writez));
OBUF_F_12 I_obuf_ram_oe 		(.O(ram_oe_n), 	.I(!oe_pipe2));
OBUF_F_12 I_obuf_ram_ce0 	(.O(ram_ce0_n), 	.I(!latched_user_ce0));
OBUF_F_12 I_obuf_ram_ce1 	(.O(ram_ce1), 		.I(latched_user_ce1));
OBUF_F_12 I_obuf_ram_ce2 	(.O(ram_ce2_n), 	.I(!latched_user_ce2));
OBUF_F_12 I_obuf_ram_adv_ld 	(.O(ram_adv_ld), 	.I(!latched_user_adv_ld));
OBUF_F_12 I_obuf_ram_wea 	(.O(ram_wea_n), 	.I(!latched_user_wea));
OBUF_F_12 I_obuf_ram_web 	(.O(ram_web_n), 	.I(!latched_user_web));
OBUF_F_12 I_obuf_ram_wec 	(.O(ram_wec_n), 	.I(!latched_user_wec));
OBUF_F_12 I_obuf_ram_wed 	(.O(ram_wed_n), 	.I(!latched_user_wed));
OBUF_F_12 I_obuf_ram_lbo 	(.O(ram_lbo), 		.I(latched_user_lbo));
OBUF_F_12 I_obuf_ram_zz 		(.O(ram_zz), 		.I(latched_user_zz));

endmodule // CTRL_BUS_INTERFACE


