// Written by George Ng ngg@ecf.utoronto.ca
// March 23, 2005
// Testbench assumes that you are processing a 48x48 size image
// If you wish to process larger images increase the image_mem size and the count stop value

`timescale 1 ns / 1 ps

module gblur_tb (
);

   `define  top_clk_per 100
   `define Max_Sim_Time 100000     // stop simulation after this time (ns)

   integer FID = $fopen("image.out"); //data output

   reg top_clk;
   reg top_reset;

   reg [7:0] pixel_reg;
   reg [11:0] count;
   // image data memory for image of 48x48 pixels
   // increase size for larger images
   reg [7:0] image_mem [2303:0];

   wire [17:0] filter_out;

   // initialize memory
   initial begin
      $readmemh("image.in", image_mem);
   end

   initial begin // generate clock
      top_clk = 0;
      forever #(`top_clk_per/2) top_clk = ~top_clk;
   end

   // memory address counter
   always@(posedge top_clk) begin
      if(top_reset == 1)
         count <= 'b0;
      else
         count <= count + 1;
   end

   // clock data into filter at every clock
   always@(posedge top_clk) begin
      if(top_reset == 1)
         pixel_reg <= 'b0;
      else
         pixel_reg <= image_mem[count];
   end

   // stop counter at last image pixel
   // assumed that input image is 48x48 pixels
   // increase count value for larger images
   always@(count) begin
      if(count == 2305) 
         $stop;
         $fclose("image.out");
   end

   // dump pixel data from simulation at every clock
   always@(posedge top_clk) begin
      if(!top_reset)
         $fdisplay(FID, "%h", filter_out);
   end

   gblur ugblur (
      .Data_out(filter_out),
      .Data_in(pixel_reg),
      .clock(top_clk),
      .reset(top_reset),
      .enable(1'b1)
   );

   // reset
   initial begin
      top_reset = 'b1;
      #250
      top_reset = 'b0;
   end



endmodule