// Written by George Ng ngg@ecf.utoronto.ca
// March 23, 2005
// Module manages the transfer of data from the FIFO to the displaycontroller
// Module writes data to the displaycontroller when user_access_ok signal is high
// FSM wakes up based on FIFO full or last_data signals

`timescale  1 ns / 1 ps

module fifo2disp_ctrl (
   clk,
   reset,
   full,
   empty,
   user_access_ok,
   last_data,
   read_en,
   data_valid
   );
   
   input  clk;
   input  reset;
   input  full;
   input  empty;
   input  user_access_ok;
   input  last_data;
   output read_en;
   output data_valid;
   
   wire clk;
   wire reset;
   wire full;
   wire empty;
   wire user_access_ok;
   wire last_data;
   reg  read_en;
   reg  data_valid;
   
   reg [1:0] y, Y; //FSM state vector
   
   parameter [1:0] IDLE = 2'b00, WAIT = 2'b01, READ = 2'b10;
   
   // Mealy Type FSM
   // Define the next state and output combinational circuit
   always@(y or full or empty or user_access_ok or last_data)
      case(y)
         IDLE :    if(full || last_data) begin
                      read_en = 1'b0;
                      Y = WAIT;
                   end
                   else begin
                      read_en = 1'b0;
                      Y = IDLE;
                   end
         WAIT :    if(user_access_ok) begin
                      read_en = 1'b1;
                      Y = READ;
                   end
                   else begin
                      read_en = 1'b0;
                      Y = WAIT;
                   end
         READ :    if(user_access_ok && !empty) begin
                      read_en = 1'b1;
                      Y = READ;
                   end
                   else if(!user_access_ok) begin
                      read_en = 1'b0;
                      Y = WAIT;
                   end
                   else if(empty) begin
                      read_en = 1'b0;
                      Y = IDLE;
                   end
         default : begin 
                      read_en = 1'b0;
                      Y = IDLE;
                   end
      endcase
      
   //Define sequential block
   always@(posedge clk)
      if(reset)
         y <= IDLE;
      else
         y <= Y;

   //Generate data valid signal
   always@(posedge clk)
      data_valid <= read_en;

      
 endmodule